/*
 * License: GPL (see COPYING) and CIPL (see COPYING-CIPL).
 *
 *
 * Originally written by Alphex Kaänoken <kaanoken@crew.org.ru>
 * All `core' and AI-like machine I wrote after beer ;-)
 * 
 * (c) Crew IT Research Labs 2004
 */
/*
 * NOTE: This is a part of X Neural Switch, parse text file, find words,
 * create xneur file for language.
 * The first string must be `[xneur-<language>]\n' where language is - 
 * us - US english
 * fi - suomi
 * ru - russian
 * ee - eesti
 */
#ifndef _ANALISYS_H_
#define _ANALISYS_H_

#include <stdlib.h>

#define MAX_LEVEL  6

/*macros */
#define TREESIZE(tree) ((tree)->size)
#define TREEROOT(tree) ((tree)->root)
#define TREEISEOB(node) ((node) == NULL)
#define TREEISLEAF(node) ((node)->left == NULL && (node)->right == NULL)
#define TREEDATA(node) ((node)->data)
#define TREELEFT(node) ((node)->left)
#define TREERIGHT(node) ((node)->right)

/*structures definion*/
typedef struct _WList {
  void *data;
  struct _WList *next;
} WList;

typedef struct _SList {
  void *data;
  struct _SList *next[MAX_LEVEL];
} SList;

struct TwiCh {
  int id;
  int point;
  char twick[5];
};

struct TriCh {
  int id;
  int point;
  char trick[10];
};

typedef struct _ProtoLang {
  char *lang;
  SList *abc;
} ProtoLang;

typedef struct _TreeNode {
  void *data;
  struct _TreeNode *left;
  struct _TreeNode *right;
} TreeNode;

typedef struct _Tree {
  int size;
  int (*compare)(const void *key1, const void *key2);
  void (*destroy)(void *data);
  TreeNode *root;
} Tree;

/*Public functions:*/
/*-base tree functions*/
void TreeInit(Tree *tree, void (*destroy)(void *data));
int TreeInsLeft(Tree *tree,TreeNode *node,const void *data);
int TreeInsRight(Tree *tree,TreeNode *node,const void *data);
void TreeRemLeft(Tree *tree,TreeNode *node);
void TreeRemRight(Tree *tree, TreeNode *node);
int TreeMerge(Tree *merge,Tree *left,Tree *right,const void *data);
/*--misc tree functions*/
int AddTreeVoid(Tree *tree,void *data);

/*-base single linked list functions*/
WList *AddWList(WList *ptr,void *data);
WList *DelWList(WList *ptr,void *data);
WList *RmWlist(WList *ptr);

/*-base sliced linked list operations*/
SList *AddSList(SList *ptr,void *data);
SList *DelSList(SList *ptr,void *data);
SList *RmSList(SList *ptr);

/*external and internal collectors functions*/
int LoadLangProto(char *flname,void *data);
int LookTextFile(char *flname,void *data);
int LookTextFileLine(char *line,void *data);

#endif
