/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <pthread.h>
#include <glib.h>

#include "interface.h"
#include "support.h"
#include "xconfig.h"
#include "xProgram.h"
#include "utils.h"
#include "error.h"
#include "trayicon.h"

xconf *xnc = NULL;	// config structure
xProgramInfo *xinfo = NULL;	// operation data structures


#define CNF_DIR     ".xneur"
#define CNF_NAME    "xneurrc"
#define RUDICT_NAME "rudict"
#define ENDICT_NAME "endict"
#define MIN_LEN_TO_WORK 2
#define LOCK_FILE   "/tmp/xneur.pid"


/*void execute (char *cmd, gboolean sync)
{
	gboolean retval;

	if (sync != FALSE)
		retval = g_spawn_command_line_sync
			(cmd, NULL, NULL, NULL, NULL);
	else
		retval = g_spawn_command_line_async (cmd, NULL);

	if (retval == FALSE)
	{
		char *msg;

		msg = g_strdup_printf
			(_("Couldn't execute command: \"<span weight=\"bold\" color=\"red\">%s</span>\"\n"
			   "Verify that this command exists.\n"),cmd);
		printf("ERROR:%s",msg);
		g_free (msg);
	}
}*/

void *xneur_main (void *data)
{
	//xconf * xnc = (xconf *)data;
	bool have_client = TRUE, to_continue;
	app_list *supported_app_list = NULL;
	size word_len = 0;
	char *config_file;
	char *rudict_file;
	char *endict_file;
	size_t cnfbnmlen;
	const char* home = getenv("HOME");
	int i;
	
	if (home == NULL)
	{
		send_error (ERROR, "Failed to get HOME variable\n", NULL);
		xProgram_exit ();
	}

	cnfbnmlen = strlen (home) + strlen (CNF_DIR) + 2; // 2 slashes will be added
	config_file = malloc (cnfbnmlen + strlen (CNF_NAME) + 1);
	rudict_file = malloc (cnfbnmlen + strlen (RUDICT_NAME) + 1);
	endict_file = malloc (cnfbnmlen + strlen (ENDICT_NAME) + 1);

	if (!config_file || !rudict_file || !endict_file)
	{
		free (config_file);
		free (rudict_file);
		free (endict_file);
		send_error (ERROR, "Failed to allocate memory\n", NULL);
		xProgram_exit ();
	}

	sprintf (config_file, "%s/%s/%s", home, CNF_DIR, CNF_NAME);
	sprintf (rudict_file, "%s/%s/%s", home, CNF_DIR, RUDICT_NAME);
	sprintf (endict_file, "%s/%s/%s", home, CNF_DIR, ENDICT_NAME); 

	// load config structure
	xnc = XConfigInit (xnc); // xconfig.c
	xnc = XConfigLoad (xnc, config_file); // xconfig.c
	xnc->rudict = XDictionaryLoad (xnc->rudict, rudict_file);
	xnc->endict = XDictionaryLoad (xnc->endict, endict_file);
	
	//init program structures
	xinfo = xProgram_init(xnc);
	if (xinfo == NULL)
		xProgram_exit ();
	
	// init current mode value
	xnc->CurrentMode = xnc->DefaultMode; 
	send_error (DEBUG, "Current mode is set to %d\n", &(xnc->CurrentMode));
	send_error (DEBUG, "Current language detect mode is set to %d\n", &(xnc->LangDetectMode));
	send_error (DEBUG, "Russian Dictionary complete\n", NULL);
	send_error (DEBUG, "English Dictionary complete\n", NULL);
	for (i=0; i<=4; i++){
		printf("[DBG]: Bind %d: key %d, modifier %d\n", i, 
						xnc->btable[i].key, 
						xnc->btable[i].key_modifier);
	}	
	while (!xProgram_isExiting ()) {
		
		// pause between iterations
		usleep (xnc->ScaningDelay);

		// refresh iteration vars
		supported_app_list = xnc->alist;

		// update current xclient info
		have_client = xProgram_update (xinfo, xnc->alist);
		if (have_client == FALSE)
			continue;

		// so client has met our conditions. Let's parse input keys
		word_len = xProgram_processInput (xinfo);

		// perform bind actions if any
		to_continue = xProgram_performCurrentBind (xinfo, xnc);
		if (to_continue)
			continue;

		// so we have no bind during this iteration. Perform mode actions
		// if any.
		if (word_len >= MIN_LEN_TO_WORK)
			xProgram_performCurrentMode (xinfo, xnc);
	}
	return NULL;
}

int
main (int argc, char *argv[])
{
	// Theads locales
	pthread_t xneur_tid;
	//xconf *xnc;
	
#ifdef ENABLE_NLS
 	setlocale(LC_ALL,"");
  	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  	textdomain (GETTEXT_PACKAGE);
#endif

	add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");

	gtk_set_locale ();
	
	/* init threads */
	g_thread_init (NULL);
	gdk_threads_init ();
	gdk_threads_enter ();
	
	gtk_init (&argc, &argv);
	
	//execute (cmd, FALSE);
	
	// 
	xnc = _malloc(sizeof(xconf));
	init_tray_icon(xnc);
	
	pthread_create (&xneur_tid, NULL, xneur_main, NULL);

	gtk_main ();
	gdk_threads_leave ();

	//sigterm(0);
  
  return 0;
}
