/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Originally written by Art Vodolazsky <burivuh@crew.org.ru>
 *  some functions, code cleanup by Alphex Kaanoken <kaanoken@crew.org.ru>
 * 
 *  (c) Crew IT Research Labs 2004
 *
 */

#include "utilities.h"
#include "error.h"
#include <string.h>
#include <stdlib.h>

//return allocated str. Don't forget to free it
char *
GetLastWord (char *string)
{
	char *res;
	if (string == NULL)
		return NULL;

	int lw = strlen (string);
	int r_lw = 0;

	/*just calculate the length of word */
	while ((--lw >= 0) && (string[lw] == ' '))
		r_lw++;
	while ((lw >= 0) && (string[lw--] != ' '))
		r_lw++;

	if (r_lw > 0)
	{
		lw = strlen (string);
		res = malloc (r_lw + 1);
		memcpy (res, (string + lw - r_lw), r_lw);
		res[r_lw] = NULLSYM;
	}
	else
		return NULL;

	return res;
}

size
numOfSymbols (char *str)
{
	size len = 0;
	num inc = 0;

	for (inc = 0; inc < strlen (str); inc++)
	{
		if (!isascii (str[inc]))
			inc++;
		len++;
	}
	return len;
}


char *
StrLastSpaceToken (char *src)
{
	char *out = NULL;
	char *tmp = NULL;
	int lw = 0, r_lw = 0, t = 0;
	if (src != NULL)
	{
		//calculate leight of last word
		t = lw = strlen ((const char *) src);
		while (src[lw] != ' ' || lw > 0)
		{
			r_lw++;
			lw--;
		}
		if (r_lw > 0)
		{
			if (r_lw == t)
			{
				out = strdup (src);
				return out;
			}
			else
			{
				out = (char *) _malloc (r_lw + 1);
				tmp = out;
				while (src[lw] != NULLSYM)
					*out++ = src[lw++];
				*out = NULLSYM;
				out = tmp;
				return out;
			}
		}
		else
			return NULL;
	}
	else
		return NULL;
}


char
get_ascii (char *symbol_str)
{
	char *rus[] =
		{ "й", "ц", "у", "к", "е", "н", "г", "ш", "щ", "з",
		  "х", "ъ",
		"ф", "ы", "в", "а", "п", "р", "о", "л", "д", "ж",
			"э",
		"я", "ч", "с", "м", "и", "т", "ь", "б", "ю",
		"Й", "Ц", "У", "К", "Е", "Н", "Г", "Ш", "Щ", "З",
			"Х", "Ъ",
		"Ф", "Ы", "В", "А", "П", "Р", "О", "Л", "Д", "Ж",
			"Э",
		"Я", "Ч", "С", "М", "И", "Т", "Ь", "Б", "Ю"
	};
	char eng[] =
		{ 'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', '[', ']',
		'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ';', '\'',
		'z', 'x', 'c', 'v', 'b', 'n', 'm', ',', '.',
		'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P', '{', '}',
		'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', ':', '"',
		'Z', 'X', 'C', 'V', 'B', 'N', 'M', '<', '>'
	};
	num iterator = 0;
	for (iterator = 0; iterator < 32; iterator++)
		if (!strcmp (symbol_str, rus[iterator]))
			return eng[iterator];

	if (!isascii (symbol_str[0]))
		return NULLSYM;

	return symbol_str[0];
}

bool
isUpperNonAlphaCyr (char symbol)
{
	num iterator;
	bool res = FALSE;
	char ch_up[] = { '"', '{', '}', ':',
		'<', '>', '!', '@',
		'#', '$', '%', '^',
		'&', '*', '(', ')',
		'_', '+', '|', '?'
	};
	for (iterator = 0; iterator < 20; iterator++)
		if (symbol == ch_up[iterator])
		{
			res = TRUE;
			break;
		}

	return res;
}

bool
isLowerNonAlphaCyr (char symbol)
{
	if ((symbol == '[') || (symbol == ']') ||
	    (symbol == ';') || (symbol == '\'') ||
	    (symbol == ',') || (symbol == '.'))
		return TRUE;
	else
		return FALSE;
}

char
getLowerCyrNonAlpha (char symbol)
{
	char sym = symbol;

	char ch_up[] = { '"', '{', '}', ':',
		'<', '>', '!', '@',
		'#', '$', '%', '^',
		'&', '*', '(', ')',
		'_', '+', '|', '?'
	};
	char ch_lo[] = { '\'', '[', ']', ';',
		',', '.', '1', '2',
		'3', '4', '5', '6',
		'7', '8', '9', '0',
		'-', '=', '\\', '/'
	};
	num iterator;

	for (iterator = 0; iterator < 20; iterator++)
		if (symbol == ch_up[iterator])
		{
			sym = ch_lo[iterator];
			break;
		}

	return sym;
}
