/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Originally written by Alphex Kaanoken <kaanoken@crew.org.ru>
 *
 *  NOTE: This is frontend for some usually used functions like malloc,
 *  it's purposes to make code good-like without many check and other
 *
 *  (c) Crew IT Research Labs 2004
 *
 */


#include "utils.h"
#include "xconfig.h"
#include "xProgram.h"
#include "error.h"
#include "about.h"
#include "interface.h"

#define CNF_DIR     ".xneur"
#define CNF_NAME    "xneurrc"

FILE *stream = NULL;

void _panic(void)
{
  fprintf(stderr,"[ERR]: Critical error.Exit.\n");
  exit(-1);
}

void *_malloc(int bytes)
{
  void *ptr=NULL;
  ptr=malloc(bytes);
  if(ptr==NULL){
    fprintf(stderr,"[ERR]: Cannot allocate %d bytes\n",bytes);
    _panic();
  }
  else
    return ptr;
}

void *_calloc(int block,int bytes)
{
  void *ptr=NULL;
  ptr=calloc(block,bytes);
  if(ptr==NULL){
    fprintf(stderr,"[ERR]: Cannot allocate %d block of %d bytes\n",block,bytes);
    _panic();
  }
  else
    return ptr;
}

void execute (char *cmd, gboolean sync)
{
	gboolean retval;

	if (sync != FALSE)
		retval = g_spawn_command_line_sync
			(cmd, NULL, NULL, NULL, NULL);
	else
		retval = g_spawn_command_line_async (cmd, NULL);

	if (retval == FALSE)
	{
		char *msg;

		msg = g_strdup_printf
			(_("Couldn't execute command: \"<span weight=\"bold\" color=\"red\">%s</span>\"\n"
			   "Verify that this command exists.\n"),cmd);
		printf("ERROR:%s",msg);
		g_free (msg);
	}
}

void xneur_exit (GtkMenuItem *menuitem,xconf *xnc)
{
	gtk_main_quit();
	exit (1);
}

void xneur_about (GtkWidget *widget,xconf *xnc)
{ 
  wAbout = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (wAbout), _("\320\236 \320\277\321\200\320\276\320\263\321\200\320\260\320\274\320\274\320\265 \"X Neural Switcher\""));
  gtk_window_set_type_hint (GTK_WINDOW (wAbout), GDK_WINDOW_TYPE_HINT_DIALOG);
  vb1About = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vb1About);
  gtk_container_add (GTK_CONTAINER (wAbout), vb1About);
	
  l2About = gtk_label_new (_("\n<b>X Neural Switcher 0.1.0</b>\n\t\t\320\260\320\262\321\202\320\276\320\274\320\260\321\202\320\270\321\207\320\265\321\201\320\272\320\270\320\271 \320\277\320\265\321\200\320\265\320\272\320\273\321\216\321\207\320\260\321\202\320\265\320\273\321\214 \320\272\320\273\320\260\320\262\320\270\320\260\321\202\321\203\321\200\321\213 \320\264\320\273\321\217 Linux\t\t"));
  gtk_widget_show (l2About);
  gtk_box_pack_start (GTK_BOX (vb1About), l2About, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (l2About), TRUE);
  gtk_label_set_justify (GTK_LABEL (l2About), GTK_JUSTIFY_CENTER);

  i2About = gtk_image_new_from_stock ("gtk-refresh", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (i2About);
  gtk_box_pack_start (GTK_BOX (vb1About), i2About, TRUE, TRUE, 0);
  gtk_widget_set_size_request (i2About, 20, 20);

  hb1About = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hb1About);
  gtk_box_pack_start (GTK_BOX (vb1About), hb1About, TRUE, TRUE, 0);

  gtk_widget_show (wAbout);
  
  btnOK = gtk_button_new ();
  g_signal_connect_swapped (G_OBJECT (btnOK), "clicked",
                              G_CALLBACK (gtk_widget_destroy),
                              G_OBJECT (wAbout));
  gtk_widget_show (btnOK);
  gtk_box_pack_end (GTK_BOX (hb1About), btnOK, FALSE, FALSE, 0);

  a1About = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (a1About);
  gtk_container_add (GTK_CONTAINER (btnOK), a1About);

  hb2About = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hb2About);
  gtk_container_add (GTK_CONTAINER (a1About), hb2About);

  i1About = gtk_image_new_from_stock ("gtk-close", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (i1About);
  gtk_box_pack_start (GTK_BOX (hb2About), i1About, FALSE, FALSE, 0);

  l1About = gtk_label_new_with_mnemonic (_("\320\227\320\260\320\272\321\200\321\213\321\202\321\214"));
  gtk_widget_show (l1About);
  gtk_box_pack_start (GTK_BOX (hb2About), l1About, FALSE, FALSE, 0);

  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (wAbout, wAbout, "wAbout");
  GLADE_HOOKUP_OBJECT (wAbout, vb1About, "vb1About");
  GLADE_HOOKUP_OBJECT (wAbout, l2About, "l2About");
  GLADE_HOOKUP_OBJECT (wAbout, i2About, "i2About");
  GLADE_HOOKUP_OBJECT (wAbout, hb1About, "hb1About");
  GLADE_HOOKUP_OBJECT (wAbout, btnOK, "btnOK");
  GLADE_HOOKUP_OBJECT (wAbout, a1About, "a1About");
  GLADE_HOOKUP_OBJECT (wAbout, hb2About, "hb2About");
  GLADE_HOOKUP_OBJECT (wAbout, i1About, "i1About");
  GLADE_HOOKUP_OBJECT (wAbout, l1About, "l1About");
	
}

gboolean save_app (GtkTreeModel *model,
                GtkTreePath  *path,
                GtkTreeIter  *iter,
                gpointer      user_data)
  {
    gchar *ptr;

    /* Note: here we use 'iter' and not '&iter', because we did not allocate
     *  the iter on the stack and are already getting the pointer to a tree iter */
    gtk_tree_model_get (GTK_TREE_MODEL(store), iter, 0, &ptr, -1);

	fputs("AddApp ", stream); 
	fputs(ptr, stream); 
	fputs("\n", stream);

    g_free(ptr);  /* the strings for us when retrieving them */

    return FALSE; /* do not stop walking the store, call us with next row */
  }

void xneur_load_preference (xconf *xnc)
{

}

void xneur_save_preference (xconf *xnc)
{
	char * app_name = "gneur";
	char *config_file;
	size_t cnfbnmlen;
	const char* home = getenv("HOME");
	
	GtkTreeIter   iter;
	char *ptr;
	int modifier;
	
 	cnfbnmlen = strlen (home) + strlen (CNF_DIR) + 2; // 2 slashes will be added
	config_file = malloc (cnfbnmlen + strlen (CNF_NAME) + 1);
	sprintf (config_file, "%s/%s/%s", home, CNF_DIR, CNF_NAME);
	
	if ((stream = fopen (config_file, "w")) != NULL) {
		
		fputs("DefaultMode ", stream); 
		ptr = g_strdup_printf("%d", gtk_combo_box_get_active (GTK_COMBO_BOX(cboxMode)));
		fputs(ptr, stream); 
		fputs("\n", stream);
		
		fputs("ConvPoint 55\n", stream); 
				
		fputs("LangDetectMode 1\n", stream); 
		
		gtk_tree_model_foreach(GTK_TREE_MODEL(store), save_app, NULL);
		
		fputs("AddBind ", stream); 	
		modifier = gtk_combo_box_get_active (GTK_COMBO_BOX(combobox4));
		if (modifier > 0) {
			modifier += 9;
		}			
		ptr = g_strdup_printf("1 %d %d", gtk_combo_box_get_active (GTK_COMBO_BOX(combobox3)), modifier);								
		fputs(ptr, stream); 
		fputs("\n", stream);
		
		fputs("AddBind ", stream); 
		modifier = gtk_combo_box_get_active (GTK_COMBO_BOX(combobox6));
		if (modifier > 0) {
			modifier += 9;
		}			
		ptr = g_strdup_printf("2 %d %d", gtk_combo_box_get_active (GTK_COMBO_BOX(combobox5)), modifier);								
		fputs(ptr, stream); 
		fputs("\n", stream);
		
		fputs("AddBind ", stream); 
		modifier = gtk_combo_box_get_active (GTK_COMBO_BOX(combobox8));
		if (modifier > 0) {
			modifier += 9;
		}			
		ptr = g_strdup_printf("3 %d %d", gtk_combo_box_get_active (GTK_COMBO_BOX(combobox7)), modifier);								
		fputs(ptr, stream); 
		fputs("\n", stream);
		
		fputs("AddBind ", stream); 
		modifier = gtk_combo_box_get_active (GTK_COMBO_BOX(combobox10));
		if (modifier > 0) {
			modifier += 9;
		}			
		ptr = g_strdup_printf("4 %d %d", gtk_combo_box_get_active (GTK_COMBO_BOX(combobox9)), modifier);								
		fputs(ptr, stream); 
		fputs("\n", stream);
		
		fclose (stream);
	}	
	// restart application with new settings
	execute(app_name, FALSE);
	gtk_main_quit();
	exit (1);		
}

void xneur_add_application ()
{
	GtkTreeIter   iter;
	FILE* fp = popen("xprop WM_CLASS", "r");
	char buffer[NAME_MAX];
	const char* cap = "WM_CLASS(STRING)";
	
	if (fp == NULL)
		return;
	if (fgets(buffer, NAME_MAX, fp) == NULL)
		return;
	if (fclose(fp))
		return;
	if (strncmp(buffer, cap, strlen(cap)) != 0)
		return;
	char* p = strstr(buffer, "\", \"");
	if (p == NULL)
		return;
	if (strlen(p += 4) < 2)
		return;
	p[strlen(p) - 2] = '\0';
	
	gtk_list_store_append(store, &iter);
  	gtk_list_store_set (store, &iter, 0, p, -1);
	
}

void xneur_rem_application ()
{
	GtkTreePath *path;
	GtkTreeIter   iter;
	GtkTreeSelection *select;	
	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview1));
	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
	if (gtk_tree_selection_get_selected (select, &store, &iter))        
	{		
		gtk_list_store_remove(GTK_LIST_STORE(store), &iter);
	}
}

void xneur_preference (GtkWidget *widget,xconf *xnc)
{
	char *config_file;
	size_t cnfbnmlen;
	const char* home = getenv("HOME");
	int modifier;
	GtkTreeIter   iter;
	
 	cnfbnmlen = strlen (home) + strlen (CNF_DIR) + 2; // 2 slashes will be added
	config_file = malloc (cnfbnmlen + strlen (CNF_NAME) + 1);
	sprintf (config_file, "%s/%s/%s", home, CNF_DIR, CNF_NAME);
	xnc = XConfigInit (xnc); // xconfig.c
	xnc = XConfigLoad (xnc, config_file);
  
  tooltips = gtk_tooltips_new ();

  wPreference = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (wPreference), _("\320\235\320\260\321\201\321\202\321\200\320\276\320\271\320\272\320\260 X Neural Switcher"));
  gtk_window_set_resizable (GTK_WINDOW (wPreference), FALSE);
	gtk_widget_show (wPreference);
	
  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (wPreference), vbox1);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox2, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox2), 2);

  label1 = gtk_label_new (_("<b>\320\240\320\265\320\266\320\270\320\274 \321\200\320\260\320\261\320\276\321\202\321\213\t\t</b>"));
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox2), label1, FALSE, FALSE, 0);
  gtk_widget_set_size_request (label1, 165, -1);
  gtk_label_set_use_markup (GTK_LABEL (label1), TRUE);

  cboxMode = gtk_combo_box_new_text ();
  gtk_widget_show (cboxMode);
  gtk_box_pack_start (GTK_BOX (hbox2), cboxMode, TRUE, TRUE, 0);
  gtk_widget_set_size_request (cboxMode, -1, 32);
  gtk_combo_box_append_text (GTK_COMBO_BOX (cboxMode), _("\320\220\320\262\321\202\320\276\320\274\320\260\321\202\320\270\321\207\320\265\321\201\320\272\320\270\320\271"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (cboxMode), _("\320\240\321\203\321\207\320\275\320\276\320\271"));
  gtk_combo_box_set_active (GTK_COMBO_BOX (cboxMode), xnc->DefaultMode);
  
  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox1), hseparator1, TRUE, TRUE, 0);
  gtk_widget_set_size_request (hseparator1, -1, 7);

  label2 = gtk_label_new (_("<b>\320\241\320\277\320\270\321\201\320\276\320\272 \320\277\321\200\320\270\320\273\320\276\320\266\320\265\320\275\320\270\320\271</b>"));
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (vbox1), label2, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label2), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label2), 0.01, 0.5);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox3, TRUE, TRUE, 3);

  scrllAppList = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrllAppList);
  gtk_box_pack_start (GTK_BOX (hbox3), scrllAppList, TRUE, TRUE, 0);
  gtk_widget_set_size_request (scrllAppList, 190, -1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrllAppList), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrllAppList), GTK_SHADOW_IN);

  treeview1 = gtk_tree_view_new ();  
  
  store = gtk_list_store_new (1, G_TYPE_STRING);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrllAppList), treeview1);
  gtk_tree_view_set_model (GTK_TREE_VIEW (treeview1), GTK_TREE_MODEL (store));
  gtk_widget_show (treeview1);  
  while (xnc->alist != NULL) {
  	gtk_list_store_append(store, &iter);
  	gtk_list_store_set (store, &iter, 0, xnc->alist->name, -1);
	xnc->alist = xnc->alist->next;
  }				  
  
  cell = gtk_cell_renderer_text_new ();					 
	
  column = gtk_tree_view_column_new_with_attributes (_("Приложение"), cell, "text", 0, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview1), GTK_TREE_VIEW_COLUMN (column));
  gtk_container_add (GTK_CONTAINER (scrllAppList), treeview1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview1), FALSE);  
  
  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (hbox3), vbox2, TRUE, TRUE, 0);

  btnAddApp = gtk_button_new_with_mnemonic (_("\320\224\320\276\320\261\320\260\320\262\320\270\321\202\321\214..."));
  gtk_widget_show (btnAddApp);
  gtk_box_pack_start (GTK_BOX (vbox2), btnAddApp, FALSE, FALSE, 0);
  g_signal_connect_swapped (G_OBJECT (btnAddApp), "clicked",
                              G_CALLBACK (xneur_add_application),
                              G_OBJECT (wPreference));
  
  btnRemApp = gtk_button_new_with_mnemonic (_("\320\243\320\264\320\260\320\273\320\270\321\202\321\214"));
  gtk_widget_show (btnRemApp);
  gtk_box_pack_start (GTK_BOX (vbox2), btnRemApp, FALSE, FALSE, 0);
  g_signal_connect_swapped (G_OBJECT (btnRemApp), "clicked",
                              G_CALLBACK (xneur_rem_application),
                              G_OBJECT (wPreference));

  hseparator2 = gtk_hseparator_new ();
  gtk_widget_show (hseparator2);
  gtk_box_pack_start (GTK_BOX (vbox1), hseparator2, TRUE, TRUE, 0);
  gtk_widget_set_size_request (hseparator2, -1, 7);

  label3 = gtk_label_new (_("<b>\320\241\320\276\321\207\320\265\321\202\320\260\320\275\320\270\321\217 \320\272\320\273\320\260\320\262\320\270\321\210</b>"));
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (vbox1), label3, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label3), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label3), 0.01, 0.5);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox4, TRUE, TRUE, 0);

  label4 = gtk_label_new (_("\320\227\320\260\320\274\320\265\320\275\320\260/\320\236\321\202\320\274\320\265\320\275\320\260 \320\260\320\262\321\202\320\276\320\267\320\260\320\274\320\265\320\275\321\213  \t"));
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (hbox4), label4, FALSE, FALSE, 0);

  combobox3 = gtk_combo_box_new_text ();
  gtk_widget_show (combobox3);
  gtk_box_pack_start (GTK_BOX (hbox4), combobox3, TRUE, TRUE, 0);
  gtk_widget_set_size_request (combobox3, 70, -1);
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox3), _("None"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox3), _("Pause/Break"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox3), _("Scroll Lock"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox3), _("Prt Sc/SysRq"));  
  gtk_combo_box_set_active (GTK_COMBO_BOX (combobox3), xnc->btable[1].key);

  combobox4 = gtk_combo_box_new_text ();
  gtk_widget_show (combobox4);
  gtk_box_pack_start (GTK_BOX (hbox4), combobox4, TRUE, TRUE, 0);
  gtk_widget_set_size_request (combobox4, 70, -1);
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox4), _("None"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox4), _("Shift"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox4), _("Control"));
  modifier = 0;
  if (xnc->btable[1].key_modifier > 0)
	  modifier = xnc->btable[1].key_modifier - 9;
  gtk_combo_box_set_active (GTK_COMBO_BOX (combobox4), modifier);

  hbox7 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox7, TRUE, TRUE, 0);

  label7 = gtk_label_new (_("\320\227\320\260\320\274\320\265\320\275\320\260 \320\277\320\276\321\201\320\273\320\265\320\264\320\275\320\265\320\271 \321\201\321\202\321\200\320\276\320\272\320\270  \t"));
  gtk_widget_show (label7);
  gtk_box_pack_start (GTK_BOX (hbox7), label7, FALSE, FALSE, 0);

  combobox5 = gtk_combo_box_new_text ();
  gtk_widget_show (combobox5);
  gtk_box_pack_start (GTK_BOX (hbox7), combobox5, TRUE, TRUE, 0);
  gtk_widget_set_size_request (combobox5, 70, -1);
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox5), _("None"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox5), _("Pause/Break"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox5), _("Scroll Lock"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox5), _("Prt Sc/SysRq"));
  gtk_combo_box_set_active (GTK_COMBO_BOX (combobox5), xnc->btable[2].key);

  combobox6 = gtk_combo_box_new_text ();
  gtk_widget_show (combobox6);
  gtk_box_pack_start (GTK_BOX (hbox7), combobox6, TRUE, TRUE, 0);
  gtk_widget_set_size_request (combobox6, 70, -1);
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox6), _("None"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox6), _("Shift"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox6), _("Control"));
  modifier = 0;
  if (xnc->btable[2].key_modifier > 0)
	  modifier = xnc->btable[2].key_modifier - 9;
  gtk_combo_box_set_active (GTK_COMBO_BOX (combobox6), modifier);  

  hbox8 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox8);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox8, TRUE, TRUE, 0);

  label8 = gtk_label_new (_("\320\241\320\274\320\265\320\275\320\260 \321\200\320\265\320\266\320\270\320\274\320\260 \321\200\320\260\320\261\320\276\321\202\321\213\t\t"));
  gtk_widget_show (label8);
  gtk_box_pack_start (GTK_BOX (hbox8), label8, FALSE, FALSE, 0);

  combobox7 = gtk_combo_box_new_text ();
  gtk_widget_show (combobox7);
  gtk_box_pack_start (GTK_BOX (hbox8), combobox7, TRUE, TRUE, 0);
  gtk_widget_set_size_request (combobox7, 70, -1);
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox7), _("None"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox7), _("Pause/Break"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox7), _("Scroll Lock"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox7), _("Prt Sc/SysRq"));
  gtk_combo_box_set_active (GTK_COMBO_BOX (combobox7), xnc->btable[3].key);

  combobox8 = gtk_combo_box_new_text ();
  gtk_widget_show (combobox8);
  gtk_box_pack_start (GTK_BOX (hbox8), combobox8, TRUE, TRUE, 0);
  gtk_widget_set_size_request (combobox8, 70, -1);
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox8), _("None"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox8), _("Shift"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox8), _("Control"));
  modifier = 0;
  if (xnc->btable[3].key_modifier > 0)
	  modifier = xnc->btable[3].key_modifier - 9;
  gtk_combo_box_set_active (GTK_COMBO_BOX (combobox8), modifier);  

  hbox9 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox9);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox9, TRUE, TRUE, 0);

  label9 = gtk_label_new (_("\320\227\320\260\320\274\320\265\320\275\320\260 \320\262\321\213\320\264\320\265\320\273\320\265\320\275\320\276\320\263\320\276 \321\202\320\265\320\272\321\201\321\202\320\260\t"));
  gtk_widget_show (label9);
  gtk_box_pack_start (GTK_BOX (hbox9), label9, FALSE, FALSE, 0);

  combobox9 = gtk_combo_box_new_text ();
  gtk_widget_show (combobox9);
  gtk_box_pack_start (GTK_BOX (hbox9), combobox9, TRUE, TRUE, 0);
  gtk_widget_set_size_request (combobox9, 70, -1);
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox9), _("None"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox9), _("Pause/Break"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox9), _("Scroll Lock"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox9), _("Prt Sc/SysRq"));
  gtk_combo_box_set_active (GTK_COMBO_BOX (combobox9), xnc->btable[4].key);

  combobox10 = gtk_combo_box_new_text ();
  gtk_widget_show (combobox10);
  gtk_box_pack_start (GTK_BOX (hbox9), combobox10, TRUE, TRUE, 0);
  gtk_widget_set_size_request (combobox10, 70, -1);
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox10), _("None"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox10), _("Shift"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (combobox10), _("Control"));
  modifier = 0;
  if (xnc->btable[4].key_modifier > 0)
	  modifier = xnc->btable[4].key_modifier - 9;
  gtk_combo_box_set_active (GTK_COMBO_BOX (combobox10), modifier);  

  hbox1 = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox1), 2);

  btnClosePref = gtk_button_new_from_stock ("gtk-close");
  gtk_widget_show (btnClosePref);
  gtk_box_pack_start (GTK_BOX (hbox1), btnClosePref, FALSE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, btnClosePref, _("\320\235\320\265 \321\201\320\276\321\205\321\200\320\260\320\275\321\217\321\202\321\214 \320\270\320\267\320\274\320\265\320\275\320\265\320\275\320\270\321\217"), NULL);
  g_signal_connect_swapped (G_OBJECT (btnClosePref), "clicked",
                              G_CALLBACK (gtk_widget_destroy),
                              G_OBJECT (wPreference));
							  
  btnOKPref = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (btnOKPref);
  gtk_box_pack_start (GTK_BOX (hbox1), btnOKPref, FALSE, TRUE, 0);
  gtk_widget_set_size_request (btnOKPref, 87, -1);
  gtk_tooltips_set_tip (tooltips, btnOKPref, _("\320\237\321\200\320\270\320\274\320\265\320\275\320\270\321\202\321\214 \320\275\320\260\321\201\321\202\321\200\320\276\320\271\320\272\320\270"), NULL);
  g_signal_connect_swapped (G_OBJECT (btnOKPref), "clicked",
                              G_CALLBACK (xneur_save_preference),
                              G_OBJECT (wPreference));
							  
  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (wPreference, wPreference, "wPreference");
  GLADE_HOOKUP_OBJECT (wPreference, vbox1, "vbox1");
  GLADE_HOOKUP_OBJECT (wPreference, hbox2, "hbox2");
  GLADE_HOOKUP_OBJECT (wPreference, label1, "label1");
  GLADE_HOOKUP_OBJECT (wPreference, cboxMode, "cboxMode");
  GLADE_HOOKUP_OBJECT (wPreference, hseparator1, "hseparator1");
  GLADE_HOOKUP_OBJECT (wPreference, label2, "label2");
  GLADE_HOOKUP_OBJECT (wPreference, hbox3, "hbox3");
  GLADE_HOOKUP_OBJECT (wPreference, scrllAppList, "scrllAppList");
  GLADE_HOOKUP_OBJECT (wPreference, vbox2, "vbox2");
  GLADE_HOOKUP_OBJECT (wPreference, btnAddApp, "btnAddApp");
  GLADE_HOOKUP_OBJECT (wPreference, btnRemApp, "btnRemApp");
  GLADE_HOOKUP_OBJECT (wPreference, hseparator2, "hseparator2");
  GLADE_HOOKUP_OBJECT (wPreference, label3, "label3");
  GLADE_HOOKUP_OBJECT (wPreference, hbox4, "hbox4");
  GLADE_HOOKUP_OBJECT (wPreference, label4, "label4");
  GLADE_HOOKUP_OBJECT (wPreference, combobox4, "combobox4");
  GLADE_HOOKUP_OBJECT (wPreference, combobox3, "combobox3");
  GLADE_HOOKUP_OBJECT (wPreference, hbox7, "hbox7");
  GLADE_HOOKUP_OBJECT (wPreference, label7, "label7");
  GLADE_HOOKUP_OBJECT (wPreference, combobox5, "combobox5");
  GLADE_HOOKUP_OBJECT (wPreference, combobox6, "combobox6");
  GLADE_HOOKUP_OBJECT (wPreference, hbox8, "hbox8");
  GLADE_HOOKUP_OBJECT (wPreference, label8, "label8");
  GLADE_HOOKUP_OBJECT (wPreference, combobox7, "combobox7");
  GLADE_HOOKUP_OBJECT (wPreference, combobox8, "combobox8");
  GLADE_HOOKUP_OBJECT (wPreference, hbox9, "hbox9");
  GLADE_HOOKUP_OBJECT (wPreference, label9, "label9");
  GLADE_HOOKUP_OBJECT (wPreference, combobox9, "combobox9");
  GLADE_HOOKUP_OBJECT (wPreference, combobox10, "combobox10");
  GLADE_HOOKUP_OBJECT (wPreference, hbox1, "hbox1");
  GLADE_HOOKUP_OBJECT (wPreference, btnClosePref, "btnClosePref");
  GLADE_HOOKUP_OBJECT (wPreference, btnOKPref, "btnOKPref");
  GLADE_HOOKUP_OBJECT_NO_REF (wPreference, tooltips, "tooltips");
	
}

void xneur_change_mode (GtkMenuItem *menuitem,xconf *xnc)
{
	printf("[DBG]: Language before switch mode - %s\n", xnc->CurrentMode);
	switch (xnc->CurrentMode) {
		case AUTO_BY_KEYLOG:
			xnc->CurrentMode = MANUAL_ON_SHORTCUT;
			break;
		case MANUAL_ON_SHORTCUT:
			xnc->CurrentMode = AUTO_BY_KEYLOG;
			break;
		default:
			break;		
	}		
	printf("[DBG]: Language after switch mode - %s\n", xnc->CurrentMode);
}
