/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Originally written by Art Vodolazsky <burivuh@crew.org.ru>
 */

#ifndef _XLIBRARY_H
#define _XLIBRARY_H

#include <X11/Xmu/Atoms.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/XKBlib.h>
#include <X11/Xutil.h>
#include <X11/Xmu/Xmu.h>

#include <time.h>

#include "types.h"

int handleXErrors (Display * d, XErrorEvent * e);

/* IMPORTANT NOTES:*/
// 1)Work only with null-terminated strings.
//   If not- correct result is not guaranteed.

/*working with xObject*/

//#define CLIENT_EVENT_MASK#define CLIENT_EVENT_MASK KeyPressMask|KeyReleaseMask|ButtonPressMask|ButtonReleaseMask|EnterWindowMask|LeaveWindowMask|PointerMotionMask|Button1MotionMask|Button2MotionMask|Button3MotionMask|Button4MotionMask|Button5MotionMask|ButtonMotionMask|KeymapStateMask|ExposureMask|VisibilityChangeMask|StructureNotifyMask|SubstructureNotifyMask|SubstructureRedirectMask|FocusChangeMask|PropertyChangeMask|ColormapChangeMask|OwnerGrabButtonMask
#define CLIENT_EVENT_MASK KeyPressMask|StructureNotifyMask
//ButtonReleaseMask|ButtonPressMask|
typedef struct {
	Display *xDisplay;
	num xScreenNum;
	Window xWindow;

} xObject;

xObject *xObject_init ();
status xObject_free (xObject * toDelete);

status xObject_connect (xObject * client);
status xObject_createWindow (xObject * toCreate);
status xObject_setXErrFunc (status (*handler) (Display *, XErrorEvent *));
Window xObject_getAppRootWindow (xObject * toGet);
Window xObject_findWindowByAtomName (xObject * toFind, char *atom_name);
Window xObject_findChildByAtom (Display * xDisplay, Window xWindow, Atom atom);
Display *xObject_getDisplay (xObject * toGet);
Window xObject_getWindow (xObject * toGet);	// check win status. return -1 - if destroyed
XClassHint *xObject_getWMClass (xObject * toGet);
char *xObject_getWMClass_class (xObject * toGet);
char *xObject_getWMClass_name (xObject * toGet);

status xObject_setEventMask (xObject * toSet, int event_mask);

/*working with xString*/

#define RUSSIAN 0
#define ENGLISH 1

#define CYR_BYTES_PER_SYM 2

typedef struct {
	char *content;		//string itself
	size currentSize;	//current size of content, keycode, keycodeModifiers fields
	KeyCode *keycode;	//array of string chars keycodes
	mask *keycodeModifiers;	//array of string chars keycodes modifiers
	xObject *home;		//current processing xObject
} xString;

xString *xString_init (size init_size);
status xString_free (xString * toDelete);

status xString_setKeycode (xString * toChange, mask keycodeModifiers);
status xString_ToASCII (xString * xstring);
	//xString is dynamic changing size structure, so use this functions to change fields
	//if unsure make init_size max possible size

status xString_setContent (xString * toChange, char *string);	//change size of all dynamic fields
status xString_catContent (xString * src, xString * dest);
status xString_addCharByIndex (xString * toChange, char sym, num index);
status xString_setCharByIndex (xString * toChange, char sym, num index);
char xString_getCharByIndex (xString * toGet, num index);
size xString_getLength (xString * toGet);

/*working with xIFocus*/
typedef struct {
	int x;
	int y;
} coords;

typedef struct {
	xObject *owner;		//input focus window
	xObject **ptr;		//pointer windows
	num ptr_num;
	status cur_status;	//input focus status
	int changed;		//TRUE if after update owner of input focus changed. By init is FALSE
	coords pointerSave;
} xIFocus;

xIFocus *xIFocus_init ();
status xIFocus_free (xIFocus * toDelete);

status xIFocus_update (xIFocus * toUpdate);

bool xIFocus_hasChanged (xIFocus * ifocus);

/*working with xEvent*/

#define CyrMask 1<<13

typedef struct {
	XEvent event;		//event to process
	xObject *communicator;	//xObject that recieves/sends event 
} xEvent;

xEvent *xEvent_init ();
status xEvent_free (xEvent * toDelete);

status xEvent_setCommunicator (xEvent * xevent, xObject * toSet);
Window xEvent_getCommunicatorWindow (xEvent * xevent);
Window xEvent_getEventWindow (xEvent * xevent);
status xEvent_updateByFocus (xEvent * xevent, xIFocus * xifocus);

status xEvent_sendXKey (xEvent * toSend, KeyCode keycode, mask keycode_modifiers);
status xEvent_sendEvent (xEvent * toSend, XEvent * event, mask event_mask);
status xEvent_sendString (xEvent * xevent, xString * xstring);

status xEvent_putBackToQueue (xEvent * toPut, XEvent * event);

bool xEvent_checkKeyRelease (xEvent * toCheck);
bool xEvent_checkKeyPress (xEvent * toCheck);
bool xEvent_checkMousePress (xEvent * toCheck);
bool xEvent_checkDestroyNotify (xEvent * toCheck);

bool xEvent_isNative (xEvent * toCheck);

char xEvent_getCurrentChar (xEvent * toGet);
KeySym xEvent_getCurrentKeySym (xEvent * toGet);
KeySym xEvent_getCurrentModifier (xEvent * toGet);
status xEvent_getCurrentLang (xEvent * toGet);
status xEvent_getSelectedText (xEvent * toGet);

KeySym xEvent_KeysymToLatin1 (KeySym ks);

KeySym xEvent_CharToKeysym (char sym);
char xEvent_KeysymToChar (KeySym ks);

/*working with xSelection*/

#define MIN_CYCLE_TIME 10

typedef struct {
	Atom selection;		//PRIMARY,SECONDARY,CLIPBOARD
	Atom target;		//"recieve" format selection atom
	Atom property;		//"recieve" selection atom
	Atom encoding;
	time_t currentTime;
	xObject *owner;		//selection owner
	xObject *requestor;	//selection requestor

} xSelection;

xSelection *xSelection_init (Display * ManagedDisplay);
status xSelection_free (xSelection * toDelete);

status xSelection_requestToConvert (xSelection * toGet);
XSelectionEvent *xSelection_waitForNotify (xSelection * toGet);
char *xSelection_getToString (xSelection * toGet, XSelectionEvent * xselection);
char *xSelection_get (xSelection * toGet);

status xSelection_setToNone (xSelection * toSet);

status xSelection_waitForSelectionOwnerIsAvailable (xSelection * toWait, num timeToWait);

typedef struct {
	xObject *xobject;
	XkbStateRec state;
	status CurrentLang;
} xSwitchLang;

xSwitchLang *xSwitchLang_init (Display * xDisplay);

status xSwitchLang_free (xSwitchLang * toDelete);

status xSwitchLang_switch (xSwitchLang * toSwitch);

//work with hotkeys

typedef struct {
	KeySym key;
	KeySym key_alt;
	KeySym modifier;
} xBinding;

typedef struct {
	xBinding *bind_list;
	num numOfBindings;
	status current_action;
} xBindingTable;

xBindingTable *xBindingTable_init (num numOfBindings);
status xBindingTable_free (xBindingTable * toFree);

status xBindingTable_setCurrentAction (xBindingTable * table, status action);
status xBindingTable_getCurrentAction (xBindingTable * table);

status xBindingTable_bindKey (xBindingTable * table, KeySym key, KeySym alt,
			      KeySym modifier, status action);

status xBindingTable_findBindAction (xBindingTable * table, KeySym key, KeySym modifier);
 
#endif /* _XLIBRARY_H */
