/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Originally written by Art Vodolazsky <burivuh@crew.org.ru>
 */



#include <X11/keysym.h>
#include <X11/Xmu/Atoms.h>

#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "xSelectManager.h"
#include "error.h"
#include "utilities.h"
status xSelectManager_getSelectedString (xString * xstring, xSelection * xselection)
{
	char *string = NULL;
	status err = SUCCESS;

	string = xSelection_get (xselection);
	if (string == NULL)
		return ERROR;
	xString_setContent (xstring, NULL);
	err = xString_setContent (xstring, string);
	if (err == BAD_ALLOC) {
		send_error (ERROR, "BAD_ALLOC!\n", NULL);
		return err;
	}

	free (string);

	return err;
}

status
xSelectManager_sendStringToFocus (xString * xstring,
				  xSwitchLang * xswitchlang, xEvent * xevent, status to_switch)
{
	XEvent release_event_backup[MAX_NATIVE_INTERRUPT_EVENTS];
	XEvent press_event_backup[MAX_NATIVE_INTERRUPT_EVENTS];
	status err = SUCCESS;
	num num_of_native_events, num_of_releases, index;
	
	xSwitchLang_switch (xswitchlang);

	switch (to_switch) {
		case RUSSIAN:
			err = xString_setKeycode (xstring, CyrMask);
			break;
		case ENGLISH:
			err = xString_setKeycode (xstring, None);
			break;
		default:
			send_error (WARNING, "Unknown language to switch!\n", NULL);
			return WARNING;
			break;
	}

	if (err != SUCCESS) {
		send_error (ERROR, "Can't code string\n", NULL);
		return ERROR;
	}

	num_of_native_events = 0;
	while (xEvent_checkKeyPress (xevent)) {
		if (xEvent_isNative (xevent)) {
			if (num_of_native_events == MAX_NATIVE_INTERRUPT_EVENTS) {
				send_error (ERROR,
					    "Too slow process. Have over %d interrupt events!\n",
					    &num_of_native_events);
				return ERROR;
			}
			release_event_backup[num_of_native_events++] = xevent->event;
		}
	}

	if (num_of_native_events > 0) {
		xSelectManager_deleteNCharsInCursorStrokeToStart (xevent, num_of_native_events);
		send_error (DEBUG, "Have interrupt native events!\n", NULL);
	}

	xEvent_sendString (xevent, xstring);

	for (index = 0; index < num_of_native_events; index++) {
		xEvent_sendEvent (xevent, &press_event_backup[index], CLIENT_EVENT_MASK);
		xEvent_putBackToQueue (xevent, &press_event_backup[index]);
	}

	return err;
}

status xSelectManager_deleteNCharsInCursorStrokeToStart (xEvent * xevent, num N)
{
	status err = SUCCESS;
	num charIndex;

/*	xSelectManager_selectNCharsInCursorStrokeToStart (xevent, N);
	err = xEvent_sendXKey (xevent,
			       XKeysymToKeycode (xevent->communicator->xDisplay, XK_Delete), None);
*/	for(charIndex = 0; charIndex < N; charIndex++){
		err = xEvent_sendXKey(xevent,
						XKeysymToKeycode(xevent->communicator->xDisplay, XK_BackSpace), None);
		if(err == ERROR){
			send_error(WARNING, "Error sending keys\n", NULL);
			err = WARNING;
			break;
		}
	}
 XFlush (xObject_getDisplay (xevent->communicator));
	return err;
}

status xSelectManager_selectAllCharsInCursorStrokeToStart (xEvent * xevent)
{
	status err = SUCCESS;
	err = xEvent_sendXKey (xevent,
			       XKeysymToKeycode (xevent->communicator->
						 xDisplay, XK_Home), ShiftMask);

	return err;
}

status xSelectManager_selectAllCharsInCursorStrokeToEnd (xEvent * xevent)
{
	status err = SUCCESS;
	err = xEvent_sendXKey (xevent,
			       XKeysymToKeycode (xevent->communicator->
						 xDisplay, XK_End), ShiftMask);
	return err;
}

status xSelectManager_selectNCharsInCursorStrokeToStart (xEvent * xevent, num N)
{
	num charIndex;
	status err = SUCCESS;

	for (charIndex = 0; charIndex < N; charIndex++) {
		err = xEvent_sendXKey (xevent,
				       XKeysymToKeycode (xevent->
							 communicator->
							 xDisplay, XK_Left), ShiftMask);
	}

	return err;
}

status xSelectManager_selectNCharsInCursorStrokeToEnd (xEvent * xevent, num N)
{
	num charIndex;
	status err = SUCCESS;

	for (charIndex = 0; charIndex < N; charIndex++) {
		err = xEvent_sendXKey (xevent,
				       XKeysymToKeycode (xevent->
							 communicator->
							 xDisplay, XK_Right), ShiftMask);
	}

	return err;
}

status xSelectManager_unselectNCharsInCursorStrokeToStart (xEvent * xevent, num N)
{
	num charIndex;
	status err = SUCCESS;

	for (charIndex = 0; charIndex < N; charIndex++) {
		err = xEvent_sendXKey (xevent,
				       XKeysymToKeycode (xevent->
							 communicator->
							 xDisplay, XK_Left), ShiftMask);
	}
	return err;
}

status xSelectManager_unselectNCharsInCursorStrokeToEnd (xEvent * xevent, num N)
{
	num charIndex;
	status err = SUCCESS;

	for (charIndex = 0; charIndex < N; charIndex++) {
		err = xEvent_sendXKey (xevent,
				       XKeysymToKeycode (xevent->
							 communicator->
							 xDisplay, XK_Right), ShiftMask);
	}

	return err;
}

status xSelectManager_unselectWholeCursorStrokeToEnd (xEvent * xevent)
{
	status err = SUCCESS;
	err = xEvent_sendXKey (xevent,
			       XKeysymToKeycode (xevent->communicator->
						 xDisplay, XK_End), ShiftMask);

	return err;
}
