/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Originally written by Alphex Kaanoken <kaanoken@crew.org.ru>
 *
 *  NOTE: is a parser header, functions prototypes and structures for X Neural Switcher 
 *
 *  (c) Crew IT Research Labs 2004
 *
 */

#ifndef _xconfig_h
#define _xconfig_h

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

#ifdef DEBUG
#define DEBUG_MSG g_print
#else /* not DEBUG */
#ifdef __GNUC__
#define DEBUG_MSG(args...)
 /**/
#else/* notdef __GNUC__ */
extern void g_none(gchar *first, ...);
#define DEBUG_MSG g_none
#endif /* __GNUC__ */
#endif /* DEBUG */

// Define "about" constant
#define AUTORS "Andrew Crew Kuznetsov, Russia\n"
#define EMAIL "andrewcrew@rambler.ru\n"
#define NAME "X Neural Switcher"
#define DESCR "It\'s program like Punto Switcher for Linux\n"
// End define "about"

#include <gtk/gtkplug.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>
	
#include "types.h"
#include "trayicon.h"
#include "eggtrayicon.h"
/* Init config values*/
#define DEFAULT_SCANNING_DELAY 20000	//microsec

typedef struct _app_list {
	int id;
	char *name;
	struct _app_list *next;
} app_list;

typedef struct _bind{
	status key;
	status key_modifier;
}bind;

typedef struct {
	EggTrayIcon *eggtray;	
	GtkTooltips *tooltip;
	GtkWidget *image;
	GtkWidget *popup_menu;
} STray;

typedef struct _xconf {
	int ConvPoint;
	status DefaultMode;
	status CurrentMode;	/*AUTO_BY_KEYLOG,MANUAL_ON_SHORTCUT */
	status LangDetectMode;  // 0 - old mode; 1 - new mode
	size ScaningDelay;	//microsecs
	app_list *alist;
	bind btable[DEFAULT_BIND_TABLE_SIZE];
	char *rudict;
	char *endict;
	STray *tray;
} xconf;

app_list *AddItem (app_list * ptr, char *name);
app_list *DelItem (app_list * ptr, int id);
app_list *FreeItems (app_list * ptr);

int XConfigCopyFile (const char *src_path, const char *dst_path, int *new_dst);

xconf *XConfigInit (xconf * ptr);
xconf *XConfigLoad (xconf * ptr, const char *fl_name);
char *XDictionaryLoad (char * ptr, const char *dict_name);
xconf *XConfigFree (xconf * ptr);

#endif //_xconfig_h
